#!/usr/bin/python

import threading, random, time

# Number of philosophers.
N = 10

# Number of times a philosopher needs to eat.
# Letting the program run forever is cute, but there isn't
# that much food in the universe <wink> .
EATME = 10

def wait(atmost=2):
    """Wait a random time, by default at most 2 seconds."""
    time.sleep(random.random() * atmost)

# Exception raised to let a philosopher know that she can't
# get both forks.
class ForksBusy(Exception):
    pass

class Philosopher(threading.Thread):
    def __init__(self, id):
        threading.Thread.__init__(self)
        self.id = id
        self.left_fork = forks[id]
        self.right_fork = forks[(id + 1) % N]
        self.feedings = 0

    def run(self):
        # Compare this main loop:  using OO organization, the
        # logic is much less cluttered, and so correspondingly
        # easier to understand and extend.

        while self.feedings < EATME:
            try:
                self.think()
                self.acquire_forks()  # may raise ForksBusy
                self.eat()
                self.release_forks()
            except ForksBusy:
                pass
        print "philosopher %d stuffed" % self.id
            
    def think(self):
        print "philosopher %d thinking" % self.id
        wait()
            
    def eat(self):
        self.feedings = self.feedings + 1
        print "philosopher %d on feeding #%d" % (self.id, self.feedings)
        wait()

    def acquire_forks(self):
        self.left_fork.acquire()
        if not self.right_fork.acquire(0):
            self.left_fork.release()
            raise ForksBusy
        
    def release_forks(self):
        self.left_fork.release()
        self.right_fork.release()

# Create the forks.
forks = []
for i in range(N):
    forks.append(threading.Lock())

# Create the philosophers, and start their contemplative lives.
philosophers = []
for i in range(N):
    p = Philosopher(i)
    philosophers.append(p)
    p.start()

# Wait for everyone to finish.
for p in philosophers:
    p.join()

print "Dinner is complete."
